"""Command-line interface."""
import io
import json
import os
from pathlib import Path

import click
import httpx

from signhost.client import RequestFixtures
from signhost.client.client import DefaultClient
from signhost.models import Signer
from signhost.models import Transaction
from signhost.models import verifications
from signhost.models.enums import Language


@click.group()
@click.version_option()
def main() -> None:
    """Signhost Api Python Client."""
    pass


@main.command()
@click.argument("filename", type=click.Path())
@click.argument("email")
@click.option("--yes", is_flag=True, help="Skip confirmation")
def transaction(
    filename: Path,
    email: str,
    yes: bool,
) -> None:
    api_key = os.getenv("SIGNHOST_API_KEY")
    app_key = os.getenv("SIGNHOST_APP_KEY")

    if not api_key or not app_key:  # pragma: no cover
        click.echo("Please set SIGNHOST_API_KEY and SIGNHOST_APP_KEY")
        return

    log_response = ResponseStorage()

    client = DefaultClient(
        api_key,
        app_key,
        event_hooks={"response": [log_response]},
    )

    response_path = Path(filename)

    signers = [
        Signer(
            SendSignRequest=False,
            Email=email,
            SignRequestMessage="Please sign this document",
            Verifications=[
                verifications.Scribble(
                    RequireHandsignature=True,
                    ScribbleNameFixed=True,
                    ScribbleName="Jelmer Draaijer",
                )
            ],
        )
    ]
    t = Transaction(
        Language=Language.NL,
        Signers=signers,
        PostbackUrl=os.getenv("TEST_POSTBACK_TUNNEL"),
    )

    transaction_created = client.transaction_init(t)

    if (
        transaction_created is None
        or transaction_created.Id is None
        or transaction_created.Signers is None
        or len(transaction_created.Signers) == 0
    ):  # pragma: no cover
        raise Exception("Transaction not created")

    client.transaction_file_put(
        transaction_created.Id,
        "file.pdf",
        io.BytesIO(b"test"),
    )
    client.transaction_start(transaction_created.Id)

    click.echo(transaction_created.Signers[0].SignUrl)

    if not yes:
        click.confirm("Are you done with signing?")  # pragma: no cover

    client.transaction_get(transaction_created.Id)
    client.transaction_file_get(transaction_created.Id, "file.pdf")
    client.transaction_cancel(transaction_created.Id)
    client.receipt_get(transaction_created.Id)

    click.echo(json.dumps(log_response.responses, indent=4))

    with response_path.open("w+") as f:
        json.dump(log_response.responses, f, indent=2)


if __name__ == "__main__":
    main(prog_name="signhost")  # pragma: no cover


class ResponseStorage:
    def __init__(self) -> None:
        self.responses: RequestFixtures = {}

    def __call__(self, response: httpx.Response) -> None:

        response.read()

        if response.headers.get("content-type", "") == "application/json":
            data = response.json()
        else:
            data = {"binary": True}

        key = str(response.url)

        if key not in self.responses:
            self.responses[key] = {}
        if response.request.method not in self.responses[key]:  # pragma: no cover
            self.responses[key][response.request.method] = {}

        self.responses[key][response.request.method][
            str(response.status_code)
        ] = data  # pragma: no cover



# flake8: noqa
from __future__ import annotations

from datetime import date
from enum import Enum
from typing import List
from typing import Literal
from typing import Optional
from typing import Union

from pydantic import BaseModel
from pydantic import Field
from typing_extensions import Annotated


class VerificationType(Enum):
    Consent = "Consent"
    DigiD = "DigiD"
    eHerkenning = "eHerkenning"
    eIDAS_Login = "eIDAS Login"
    iDeal = "iDeal"
    iDIN = "iDIN"
    itsme_Identification = "itsme Identification"
    PhoneNumber = "PhoneNumber"
    Scribble = "Scribble"
    itsme_sign = "itsme sign"
    SigningCertificate = "SigningCertificate"
    SURFnet = "SURFnet"
    ZealiD_Qualified = "ZealiD Qualified"
    IPAddress = "IPAddress"


class Betrouwbaarheidsniveau(Enum):
    Basis = "Basis"
    Midden = "Midden"
    Substantieel = "Substantieel"
    Hoog = "Hoog"


class Verification(BaseModel):
    pass


class Consent(Verification):
    Type: Literal["Consent"] = "Consent"


class DigiD(Verification):
    Type: Literal["DigiD"] = "DigiD"
    Bsn: Optional[str] = Field(
        None,
        description="When provided, the provided value must match the BSN of the credentials returned by DigiD.\nThe BSN is required to match an '11-proef'.\n",
        # noqa
    )
    Betrouwbaarheidsniveau: Optional[Betrouwbaarheidsniveau] = Field(
        None,
        description="The level of confidence with which the identity of the signer has been determined.\nFor further information, please refer to [Logius](https://www.logius.nl/diensten/digid/hoe-werkt-het).\n",
        # noqa
    )


class EHerkenning(Verification):
    Type: Literal["eHerkenning"] = "eHerkenning"
    Uid: Optional[str] = None
    EntityConcernIdKvkNr: Optional[str] = Field(
        None,
        description="When provided, the provided value must match the KvK number returned by eHerkenning.\n",
    )


class IDeal(Verification):
    Type: Literal["iDeal"] = "iDeal"
    Iban: Optional[str] = Field(
        None,
        description="The IBAN of the signer.\nWhen provided during the creation of the transaction this IBAN is\nverified during the verification flow to make sure these and the actual IBAN number match.\n",
        # noqa
        example="NL13TEST0123456789",
    )
    AccountHolderName: Optional[str] = None
    AccountHolderCity: Optional[str] = None


class IDIN(Verification):
    Type: Literal["iDIN"] = "iDIN"
    AccountHolderName: Optional[str] = Field(
        None,
        description="Name of the idin consumer / signer.\nCurrently we don't support supplying a value in this property to ensure the expected account holder name matches.\nThis could change in the future.\n",
    )
    AccountHolderAddress1: Optional[str] = None
    AccountHolderAddress2: Optional[str] = None
    AccountHolderDateOfBirth: Optional[date] = Field(
        None,
        description="Date of birth of idin consumer / signer",
        example="2001-12-31",
    )
    Attributes: Optional[List[str]] = Field(
        None,
        description="Contains all available iDIN attributes.\nThese attributes may change, therefore we cannot guarantee the availability of any of these attributes.\n",
        # noqa
    )


class IPAddress(Verification):
    Type: Literal["IPAddress"] = "IPAddress"
    IPAddress: Optional[str] = None


class PhoneNumber(Verification):
    Type: Literal["PhoneNumber"] = "PhoneNumber"
    Number: Optional[str] = Field(
        None,
        description="The mobile phone number of the signer.\n"
        "Must conform to E.164,\n"
        "[the international public telecommunication numbering plan](https://en.wikipedia.org/wiki/E.164),\n"
        "which requires the country calling code (e.g. +31).\n",
        example="+31123456789",
    )


class Scribble(Verification):
    Type: Literal["Scribble"] = "Scribble"
    RequireHandsignature: Optional[bool] = Field(
        False,
        description="When set the signer is required to draw a hand signature,\n"
        "either via computer mouse, trackpad, or touchscreen.\n",
    )
    ScribbleNameFixed: Optional[bool] = Field(
        False,
        description="When set the signer will not be able to change its scribble name.\n"
        "When not set the signer can correct or provide a scribble name.\n",
    )
    ScribbleName: Optional[str] = Field(
        None,
        description="The name of the signer, this will be pre filled in the scribble form.\n"
        "Required if `ScribbleNameFixed` is set.\n",
    )


class SigningCertificate(Verification):
    Type: Literal["SigningCertificate"] = "SigningCertificate"
    Issuer: Optional[str] = None
    Subject: Optional[str] = None
    Thumbprint: Optional[str] = None


class SURFnet(Verification):
    Type: Literal["SURFnet"] = "SURFnet"
    Uid: Optional[str] = None
    Attributes: Optional[List[str]] = Field(
        None,
        description="Contains all available SURFnet attributes.\n"
        "These attributes may change, therefore we cannot guarantee "
        "the availability of any of these attributes.\n",
    )


VerificationAnnotatedType = Annotated[
    Union[
        Consent,
        DigiD,
        EHerkenning,
        IDeal,
        IDIN,
        IPAddress,
        PhoneNumber,
        Scribble,
        SigningCertificate,
        SURFnet,
    ],
    Field(discriminator="Type"),
]



import io
from json import JSONDecodeError
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

import httpx

from .. import models
from . import errors


_T = TypeVar("_T")


class BaseClient:
    """Class with method shared between async and default client"""

    ERROR_RESPONSE_MAPPING: Dict[int, Type[errors.SignhostError]] = {
        422: errors.SignhostValidationError,
        401: errors.SignhostAuthenticationError,
        403: errors.SignhostAuthenticationError,
        404: errors.SignhostNotFoundError,
        400: errors.SignhostValidationError,
        500: errors.SignhostServerError,
    }

    def __init__(
        self,
        api_key: str,
        app_key: str,
        base_url: str = "https://api.signhost.com/api/",
    ):
        self.api_key = api_key
        self.app_key = app_key
        self.base_url = base_url

    def process_response(
        self, response: httpx.Response, model: Type[_T], status_code_success: int = 200
    ) -> _T:

        try:
            if response.status_code == status_code_success:
                response_json = response.json()
                return model(**response_json)
            else:
                raise self.create_error(response)
        except JSONDecodeError as e:
            raise errors.SignhostServerError(
                "Invalid json from server", status_code=400
            ) from e

    def create_error(self, response: httpx.Response) -> errors.SignhostError:

        try:
            response_json = response.json()
        except JSONDecodeError:
            response_json = {"message": response.text}

        response_json["status_code"] = response.status_code

        exception_type = self.map_exception(response)
        return exception_type(
            status_code=response.status_code,
            json=response_json,
            message="Error from server",
        )

    def map_exception(self, response: httpx.Response) -> Type[errors.SignhostError]:
        exception_type = self.ERROR_RESPONSE_MAPPING.get(
            response.status_code, errors.SignhostError
        )
        return exception_type

    def authenticate_request(self, request: httpx.Request) -> httpx.Request:

        request.headers["Authorization"] = f"APIKey {self.api_key}"
        request.headers["Application"] = f"APPKey {self.app_key}"

        return request


class DefaultClient(BaseClient):
    """Synchronous client"""

    client: httpx.Client

    def __init__(
        self,
        api_key: str,
        app_key: str,
        base_url: str = "https://api.signhost.com/api/",
        **httpx_kwargs: Any,
    ):
        super().__init__(api_key, app_key, base_url)
        self.client = self.create_client(base_url, **httpx_kwargs)

    def create_client(self, base_url: str, **httpx_kwargs: Any) -> httpx.Client:
        httpx_kwargs.setdefault("base_url", base_url)
        httpx_kwargs.setdefault("auth", self.authenticate_request)
        return httpx.Client(**httpx_kwargs)

    def transaction_get(self, transaction_id: str) -> models.Transaction:
        """GET /api/transaction/{transactionId}"""
        response = self.client.get(f"transaction/{transaction_id}")

        return self.process_response(response, models.Transaction)

    def transaction_cancel(
        self, transaction_id: str, send_notifications: bool = False, reason: str = ""
    ) -> models.Transaction:
        """DELETE /api/transaction/{transactionId}"""
        response = self.client.request(
            "DELETE",
            f"transaction/{transaction_id}",
            json={"SendNotifications": send_notifications, "Reason": reason},
        )
        return self.process_response(response, models.Transaction)

    def transaction_file_get(self, transaction_id: str, file_id: str) -> bytes:
        """
        GET /api/transaction/{transactionId}/file/{fileId}

        Returns the contents of the (signed) document(s).
        """
        response = self.client.get(f"transaction/{transaction_id}/file/{file_id}")

        if response.status_code == httpx.codes.OK:
            return response.content

        raise self.create_error(response)

    def receipt_get(self, transaction_id: str) -> bytes:
        """
        GET /api/file/receipt/{transactionId}

        Returns the contents of the receipt when the transaction is successfully signed (Status=30)
        """
        response = self.client.get(f"/api/file/receipt/{transaction_id}")

        if response.status_code == httpx.codes.OK:
            return response.content

        raise self.create_error(response)

    def transaction_init(self, transaction: models.Transaction) -> models.Transaction:
        """POST /api/transaction"""

        data = transaction.json()
        content_length = str(len(data))
        content_type = "application/json"
        headers = {"Content-Length": content_length, "Content-Type": content_type}

        response = self.client.post("transaction", content=data, headers=headers)

        return self.process_response(response, models.Transaction)

    def transaction_file_put(
        self, transaction_id: str, file_id: str, file_content: io.IOBase
    ) -> bool:
        """PUT /api/transaction/{transactionId}/file/{fileId}"""
        # sha = hashlib.sha256(file_content.read())
        # file_digest = base64.urlsafe_b64encode(sha.digest()).decode("utf-8")

        file_content.seek(0)

        response = self.client.put(
            f"transaction/{transaction_id}/file/{file_id}",
            content=file_content,
            headers={
                "Content-Type": "application/pdf",
                # "Digest": f"SHA256={file_digest}",
            },
        )

        if response.status_code in [
            httpx.codes.CREATED,
            httpx.codes.ACCEPTED,
            httpx.codes.NO_CONTENT,
        ]:
            return True

        raise self.create_error(response)

    def transaction_start(self, transaction_id: str) -> bool:
        """PUT /api/transaction/{transactionId}/start"""
        response = self.client.put(f"transaction/{transaction_id}/start")

        if response.status_code != httpx.codes.NO_CONTENT:
            raise self.create_error(response)

        return True


class AsyncClient(BaseClient):
    """Asynchronous client."""

    client: httpx.AsyncClient



# flake8: noqa
# generated by datamodel-codegen:
#   filename:  signhost_openapi.yml
#   timestamp: 2023-02-09T16:34:43+00:00

from __future__ import annotations

from datetime import date
from datetime import datetime
from typing import Any
from typing import Dict
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field

from signhost.models.enums import Betrouwbaarheidsniveau1
from signhost.models.enums import Code
from signhost.models.enums import FormSetType
from signhost.models.enums import Language
from signhost.models.enums import Rel
from signhost.models.enums import Status
from signhost.models.enums import Type
from signhost.models.verifications import Verification
from signhost.models.verifications import VerificationAnnotatedType


class Activity(BaseModel):
    Id: Optional[str] = None
    Code: Optional[Code] = Field(
        None,
        description="* 101 -\tInvitation sent\n"
        "* 102 -\tReceived\n"
        "* 103 -\tOpened\n"
        "* 104 -\tReminder sent\n"
        "* 105 -\tDocument opened, Info property contains the file id of the opened document.\n"
        "* 201 -\tCancelled\n"
        "* 202 -\tRejected\n"
        "* 203 -\tSigned\n"
        "* 301 -\tSigned document sent\n"
        "* 302 -\tSigned document opended\n"
        "* 303 -\tSigned document downloaded\n"
        "* 401 -\tReceipt sent\n"
        "* 402 -\tReceipt opened\n"
        "* 403 -\tReceipt downloaded\n",
    )
    Info: Optional[str] = Field(
        None,
        description="May contain additional information belonging to this activity",
    )
    CreatedDateTime: Optional[datetime] = None


class Receiver(BaseModel):
    Name: str = Field(..., description="The name of the receiver.")
    Email: str = Field(..., description="The e-mail address of the reveiver.")
    Language: Optional[str] = Field(
        "nl-NL",
        description="The language of the receiver, only de-DE, en-US, es-ES, fr-FR, it-IT and nl-NL are allowed.",
    )
    Subject: Optional[str] = Field(
        None,
        description="The subject of the receiver email in plain text.\n"
        "Maximum of 64 characters allowed.\n"
        "Omitting this parameter will enable the default subject.\n",
    )
    Message: str = Field(
        ...,
        description="The email message towards the receiver in plain text."
        " Newlines can be created by including a \\n in the json, HTML is not allowed.",
    )
    Reference: Optional[str] = Field(None, description="The reference of the receiver.")
    Context: Optional[Dict[str, Any]] = Field(
        None,
        description="Any valid json object which we will return back to you when "
        "doing a GET on the transaction or when we send a postback.",
    )


class Authentication(BaseModel):
    Type: Type = Field(
        ...,
        description="Type of the authentication object.\n"
        "The `Type` property **must** be the first property in the json!\n\n"
        "The order in which the authentications are provided determine "
        "in which order the signer will have to perform the specified method.\n",
    )


class Link(BaseModel):
    Rel: Optional[Rel] = Field(None, description="The type of file you can download.")
    Type: Optional[str] = Field(
        None,
        description="The type of the file you can expect to download. "
        "Include this in your 'Accept' header when requesting the file.",
    )
    Link: Optional[str] = Field(
        None, description="Url containing the link to the file."
    )


class Signers(BaseModel):
    FormSets: Optional[List[str]] = Field(
        None, description="List of formset keys to be assigned to this signer."
    )


class Location(BaseModel):
    Search: Optional[str] = Field(
        None,
        description="The text to search in the pdf document to use as "
        "the position for the field. For example `{{Signer1}}`.",
    )
    Occurence: Optional[int] = Field(
        None, description="When using text search, only match this matched occurence."
    )
    Top: Optional[int] = Field(
        None, description="Offset from the top of the search text or the page"
    )
    Right: Optional[int] = Field(
        None, description="Offset from the right of the search or the page"
    )
    Bottom: Optional[int] = Field(
        None, description="Offset from the bottom of the search or the page"
    )
    Left: Optional[int] = Field(
        None, description="Offset from the left of the search or the page"
    )
    Width: Optional[int] = Field(
        None,
        description="The width of the field, can’t be used when both Left and Right are specified.\n"
        "For signature and seal fields we suggest a width of 140.\n",
    )
    Height: Optional[int] = Field(
        None,
        description="The height of the field, can’t be used when both Bottom and Top are specified.\n"
        "For signature and seal fields we suggest a height of 70.\n",
    )
    PageNumber: Optional[int] = Field(
        None, description="On which page the field should be placed."
    )


class FormSets(BaseModel):
    Type: Optional[FormSetType] = Field(
        None,
        description="Field type to create.\n"
        "\n"
        "* Seal is not yet implemented, this will specify the properties of a seal.\n"
        "* Signature, specifies a signature field\n"
        "* Check, specifies a checkbox. You'll have to set the `value` property\n"
        "* SingleLine, specifies a single line textbox\n",
    )
    Location: Optional[Location] = Field(
        None,
        description="Specify where the field should be placed within the document.",
    )


class FileMetaData(BaseModel):
    DisplayOrder: Optional[int] = Field(
        None, description="With what order number we'll display the file to the signer"
    )
    DisplayName: Optional[str] = Field(
        None, description="With what name we'll display the file to the signer"
    )
    SetParaph: Optional[bool] = Field(
        None,
        description=(
            "Places a copy of the signer's scribble image on the bottom right of every page where no signature is "
            "present.\n"
            "Note: due to the nature of advanced or qualified digital signatures, paraphs are merely a cosmetic "
            "addition.\n"
        ),
    )
    Signers: Optional[Dict[str, Signers]] = Field(
        None,
        description="Map of array of formsets.\nEach key should be a valid signer id.\n",
    )
    FormSets: Optional[Dict[str, Dict[str, FormSets]]] = Field(
        None,
        description="Map of one or more form set definitions.\n"
        "The key of the map will be the formset name.\n"
        "The value will be the formset definition\n",
    )


class ErrorModel(BaseModel):
    Message: Optional[str] = Field(
        None, description="Message describing the error in the request."
    )


class DigiDAuthentication(Authentication):
    Bsn: Optional[str] = Field(
        None,
        description="The provided value must match the BSN of the credentials returned by DigiD.\n"
        "The BSN is required to match an '11-proef'.\n",
    )
    Betrouwbaarheidsniveau: Optional[Betrouwbaarheidsniveau1] = Field(
        None,
        description="The level of confidence with which the identity of the signer has been determined.\n"
        "For further information, please refer to [Logius](https://www.logius.nl/diensten/digid/hoe-werkt-het).\n",
    )


class PhoneNumberAuthentication(Authentication):
    Number: Optional[str] = Field(
        None,
        description="The mobile phone number of the signer.\n"
        "Must conform to E.164,\n"
        "[the international public telecommunication numbering plan](https://en.wikipedia.org/wiki/E.164),\n"
        "which requires the country calling code (e.g. +31).\n",
        example="+31123456789",
    )


class EIDASLogin(Verification):
    Uid: Optional[str] = Field(
        None, description="The unique identifier returned by eIDAS Login."
    )
    Level: Optional[str] = Field(None, description="The Level of Assurance.")
    FirstName: Optional[str] = Field(
        None, description="The first name of the signer as returned by eIDAS Login."
    )
    LastName: Optional[str] = Field(
        None, description="The last name of the signer as returned by eIDAS Login."
    )
    DateOfBirth: Optional[date] = Field(
        None, description="The date of birth of the signer as returned by eIDAS Login."
    )
    Attributes: Optional[List[str]] = Field(
        None,
        description="Contains all available eIDAS Login attributes.\n"
        "These attributes may change, therefore we cannot guarantee the availability of any of these attributes.\n",
    )


class ItsmeIdentification(Verification):
    PhoneNumber: Optional[str] = Field(
        None,
        description="The mobile phone number of the signer.\n"
        "Must be conform E.164,\n"
        "[the international public telecommunication numbering plan](https://en.wikipedia.org/wiki/E.164),\n"
        "which requires the country calling code (Only the Belgian country calling code is supported: +32).\n",
        example="+32123456789",
    )
    Attributes: Optional[List[str]] = Field(
        None,
        description="Contains all available itsme Identification attributes.\n"
        "These attributes may change, therefore we cannot guarantee the availability of any of these attributes.\n",
    )


class ItsmeSign(Verification):
    Issuer: Optional[str] = None
    Subject: Optional[str] = None
    Thumbprint: Optional[str] = None


class ZealiDQualified(Verification):
    Issuer: Optional[str] = None
    Subject: Optional[str] = None
    Thumbprint: Optional[str] = None


class Signer(BaseModel):
    Id: Optional[str] = Field(
        None,
        description="The id of the signer, must be unique within a transaction.\n"
        "If you don't provide an id we will generate one for you.\n",
    )
    Email: str = Field(
        ...,
        description="The e-mail address of the signer",
        example="john.doe@example.com",
    )
    IntroText: Optional[str] = Field(
        None,
        description="An intro text to show to the user during the sign proces.\n"
        "This will be shown on the first screen to the signer and supports limitted markdown markup.\n\n"
        "The following markup is supported:"
        "\n- `# Headings`"
        "\n- `*Emphasis*` / `_Emphasis_`"
        "\n- `**Stong**` / `__Strong__`"
        "\n- `1. Ordered` and `- Unordered` lists\n",
    )
    Authentications: Optional[List[Authentication]] = Field(
        None,
        description="List of authentications that the signer has to authenticate with.\n"
        "The order in which the authentications are provided determine in which order "
        "the signer will have to perform the specified method.\n\n"
        "Authentications must be performed before the document(s) can be viewed.\n\n"
        "You **must** explicitly specify the API-version when using this feature.\n"
        "This is done with the header: 'Accept: application/vnd.signhost.v1+json'.\n",
    )
    Verifications: Optional[List[VerificationAnnotatedType]] = Field(
        None,
        description="List of verifications that the signer has to verify with.\n"
        "The order in which the verifications are provided determine in "
        "which order the signer will have to perform the specified method.\n\n"
        "Verifications must be performed before the document(s) can be signed.\n\n"
        "You **must** use one of the following verifications as the last method:"
        "\n- Consent\n"
        "- itsme sign*\n"
        "- PhoneNumber\n"
        "- Scribble\n"
        "- SigningCertificate*\n"
        "- ZealiD Qualified*\n\n"
        "* These verifications can not be used in any other position than the last.\n",
    )
    SendSignRequest: Optional[bool] = Field(
        True, description="Send a sign invitation to the signer his e-mail address."
    )
    SignUrl: Optional[str] = Field(
        None,
        description="A unique URL per signer that provides the signing flow for the signer."
        "\nAvailable / valid if `SendSignRequest` is set to false.\n",
    )
    SignRequestSubject: Optional[str] = Field(
        None,
        description="The subject of the sign request email in plain text."
        "\nMaximum of 64 characters allowed."
        "\nOmitting this parameter will enable the default subject.\n",
    )
    SignRequestMessage: Optional[str] = Field(
        None,
        description="The message of the sign request in plain text."
        "\nNewlines can be created by including a \\n in the json, HTML is not allowed."
        "\nRequired if `SendSignRequest` is true\n",
    )
    SendSignConfirmation: Optional[bool] = Field(
        None,
        description="Send the sign confirmation to the signer his e-mail address."
        "\nDefault value is the value of `SendSignRequest`\n",
    )
    Language: Optional[Language] = Field(
        "nl-NL",
        description="The language of the receiving user, "
        "only de-DE, en-US, es-ES, fr-FR, it-IT, pl-PL and nl-NL are allowed.",
    )
    ScribbleName: Optional[str] = Field(
        None,
        description="The name of the signer, this will be pre filled in the scribble form.",
    )
    DaysToRemind: Optional[int] = Field(
        7,
        description="Amount of days before reminding the signers. -1 to disable reminders.\n"
        "Ignored if `SendSignRequest` is set to false."
        "\nBy default your organisation's setting will be used.\n",
    )
    Expires: Optional[datetime] = Field(
        None,
        description="When set the signer is no longer allowed to sign the transaction after this date.",
    )
    Reference: Optional[str] = Field(None, description="The reference of the signer.")
    RejectReason: Optional[str] = Field(
        None,
        description="The rejection reason that was given by the signer when the transaction was rejected.",
    )
    ReturnUrl: Optional[str] = Field(
        "https://signhost.com",
        description="The url to redirect the user to after signing, rejecting or cancelling.",
    )
    Context: Optional[Dict[str, Any]] = Field(
        None,
        description="Any valid json object which we will return back to you"
        " when doing a GET on the transaction or when we send a postback.",
    )
    Activities: Optional[List[Activity]] = Field(
        None,
        description="List of activities attached to this signer.\n"
        "Activities are added by signhost when a signer event occured.\n",
    )


class FileEntry(BaseModel):
    Links: Optional[List[Link]] = None
    DisplayName: Optional[str] = Field(
        None,
        description="The name of the document that was displayed to the user while signing the documents.",
    )


class Transaction(BaseModel):
    Id: Optional[str] = Field(
        None,
        description="The id of the transaction.\n"
        "Currently this property is read only but this may change in the future.\n",
    )
    Files: Optional[Dict[str, FileEntry]] = Field(
        None, description="A map of files attached to this transaction."
    )
    Language: Optional[Language] = Field(
        None,
        description="The language of the sender notifications and the receipt,"
        " only de-DE, en-US, es-ES, fr-FR, it-IT, pl-PL and nl-NL are allowed.",
    )
    Seal: Optional[bool] = Field(
        False, description="Seal the document before sending to the signers."
    )
    Signers: Optional[List[Signer]] = Field(None, description="The signer information.")
    Receivers: Optional[List[Receiver]] = None
    Reference: Optional[str] = Field(
        None, description='The reference of the transaction. For example "1234"'
    )
    PostbackUrl: Optional[str] = Field(
        None,
        description="The absolute url to postback the status updates. For example https://example.com/postback.php",
    )
    SignRequestMode: Optional[int] = Field(
        2,
        description="Set to 1 for sending at once, to 2 for sequential.\n"
        "Ignored if `SendSignRequest` is set to false.\n",
    )
    DaysToExpire: Optional[int] = Field(
        60, description="Amount of days before expiration. Max 90 days."
    )
    SendEmailNotifications: Optional[bool] = Field(
        True, description="Send e-mail notifications to the sender."
    )
    Status: Optional[Status] = Field(
        None,
        description="Current transaction status.\n\n"
        "* 5 - Waiting for document\n"
        "* 10 - Waiting for signer\n"
        "* 20 - In progress\n"
        "* 30 - Signed (end state)\n"
        "* 40 - Rejected (end state)\n"
        "* 50 - Expired (end state)\n"
        "* 60 - Cancelled (end state)\n"
        "* 70 - Failed (end state)\n",
    )
    CancelationReason: Optional[str] = Field(
        None, description="The original cancellation reason given during a DELETE call."
    )
    Context: Optional[Dict[str, Any]] = Field(
        None,
        description="Any valid json object which we will return back to you "
        "when doing a GET on the transaction or when we send a postback.",
    )



from typing import Any
from typing import Dict
from typing import Optional

from attrs import define


@define
class SignhostError(Exception):
    message: str
    status_code: int
    json: Optional[Dict[str, Any]] = None


class SignhostValidationError(SignhostError):
    pass


class SignhostAuthenticationError(SignhostError):
    pass


class SignhostNotFoundError(SignhostError):
    pass


class SignhostServerError(SignhostError):
    pass



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['signhost', 'signhost.client', 'signhost.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=22.2.0,<23.0.0',
 'click>=8.0.1',
 'httpx>=0.23.3,<0.24.0',
 'pydantic>=1.10.4,<2.0.0',
 'pyupgrade==3.3.1']

entry_points = \
{'console_scripts': ['signhost = signhost.__main__:main']}

setup_kwargs = {
    'name': 'signhost-api-python-client',
    'version': '0.1.4',
    'description': 'Signhost Api Python Client',
    'long_description': '# Signhost Api Python Client\n\n[![PyPI](https://img.shields.io/pypi/v/signhost-api-python-client.svg)][pypi_]\n[![Status](https://img.shields.io/pypi/status/signhost-api-python-client.svg)][status]\n[![Python Version](https://img.shields.io/pypi/pyversions/signhost-api-python-client)][python version]\n[![License](https://img.shields.io/pypi/l/signhost-api-python-client)][license]\n\n[![Read the documentation at https://signhost-api-python-client.readthedocs.io/](https://img.shields.io/readthedocs/signhost-api-python-client/latest.svg?label=Read%20the%20Docs)][read the docs]\n[![Tests](https://github.com/foarsitter/signhost-api-python-client/workflows/Tests/badge.svg)][tests]\n[![Codecov](https://codecov.io/gh/foarsitter/signhost-api-python-client/branch/main/graph/badge.svg)][codecov]\n\n[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)][pre-commit]\n[![Black](https://img.shields.io/badge/code%20style-black-000000.svg)][black]\n\n[pypi_]: https://pypi.org/project/signhost-api-python-client/\n[status]: https://pypi.org/project/signhost-api-python-client/\n[python version]: https://pypi.org/project/signhost-api-python-client\n[read the docs]: https://signhost-api-python-client.readthedocs.io/\n[tests]: https://github.com/foarsitter/signhost-api-python-client/actions?workflow=Tests\n[codecov]: https://app.codecov.io/gh/foarsitter/signhost-api-python-client\n[pre-commit]: https://github.com/pre-commit/pre-commit\n[black]: https://github.com/psf/black\n\n## Features\n\n- 100% test coverage\n\n## Requirements\n\n- httpx\n- pydantic\n- attr\n- click\n\n## Installation\n\nYou can install _Signhost Api Python Client_ via [pip] from [PyPI]:\n\n```console\n$ pip install signhost-api-python-client\n```\n\n## Usage\n\n```python\nimport io\nfrom signhost import models\nfrom signhost.client import DefaultClient\n\nsignhost = DefaultClient(api_key="str", app_key="str")\ntransaction = models.Transaction(signers=[models.Signer(email="str")])\n\ntransaction = signhost.transaction_init(transaction=transaction)\nsignhost.transaction_file_put(\n    transaction.Id,\n    "file.pdf",\n    io.BytesIO(b"test"),\n)\ntransaction = signhost.transaction_start(transaction.Id)\n\nprint("Sign the contract over here", transaction.Signers[0].SignUrl)\n\nsignhost.transaction_get(transaction.Id)\nsignhost.transaction_file_get(transaction.Id, "file.pdf")\nsignhost.receipt_get(transaction.Id)\n```\n\nPlease see the [Command-line Reference] for details.\n\n## Contributing\n\nContributions are very welcome.\nTo learn more, see the [Contributor Guide].\n\n## License\n\nDistributed under the terms of the [MIT license][license],\n_Signhost Api Python Client_ is free and open source software.\n\n## Issues\n\nIf you encounter any problems,\nplease [file an issue] along with a detailed description.\n\n## Credits\n\nThis project was generated from [@cjolowicz]\'s [Hypermodern Python Cookiecutter] template.\n\n[@cjolowicz]: https://github.com/cjolowicz\n[pypi]: https://pypi.org/\n[hypermodern python cookiecutter]: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n[file an issue]: https://github.com/foarsitter/signhost-api-python-client/issues\n[pip]: https://pip.pypa.io/\n\n<!-- github-only -->\n\n[license]: https://github.com/foarsitter/signhost-api-python-client/blob/main/LICENSE\n[contributor guide]: https://github.com/foarsitter/signhost-api-python-client/blob/main/CONTRIBUTING.md\n[command-line reference]: https://signhost-api-python-client.readthedocs.io/en/latest/usage.html\n',
    'author': 'Jelmer Draaijer',
    'author_email': 'info@jelmert.nl',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/foarsitter/signhost-api-python-client',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from enum import Enum


class Betrouwbaarheidsniveau1(Enum):
    Basis = "Basis"
    Midden = "Midden"
    Substantieel = "Substantieel"
    Hoog = "Hoog"


class Code(Enum):
    integer_101 = 101
    integer_102 = 102
    integer_103 = 103
    integer_104 = 104
    integer_105 = 105
    integer_201 = 201
    integer_202 = 202
    integer_203 = 203
    integer_301 = 301
    integer_302 = 302
    integer_303 = 303
    integer_401 = 401
    integer_402 = 402
    integer_403 = 403


class Language(Enum):
    DE = "de-DE"
    US = "en-US"
    ES = "es-ES"
    FR = "fr-FR"
    IT = "it-IT"
    PL = "pl-PL"
    NL = "nl-NL"


class Rel(Enum):
    file = "file"
    receipt = "receipt"
    signer_sign = "signer.sign"
    signer_download = "signer.download"


class Status(Enum):
    WAITING_FOR_DOCUMENT = 5
    WAITING_FOR_SIGNER = 10
    IN_PROGRESS = 20
    SIGNED = 30
    REJECTED = 40
    EXPIRED = 50
    CANCELLED = 60
    FAILED = 70


class Type(Enum):
    DigiD = "DigiD"
    PhoneNumber = "PhoneNumber"


class FormSetType(Enum):
    Seal = "Seal"
    Signature = "Signature"
    Check = "Check"
    SingleLine = "SingleLine"
